@@ -2,7 +2,6 @@ package Git::PurePerl::NewObject;
 use Moose;
 use MooseX::StrictConstructor;
 use Moose::Util::TypeConstraints;
-use namespace::autoclean;
 
 enum 'ObjectKind' => qw(commit tree blob tag);
 
@@ -11,6 +10,8 @@ has 'size' => ( is => 'ro', isa => 'Int', required => 0, lazy_build => 1 );
 has 'content' => ( is => 'rw', isa => 'Str', required => 0, lazy_build => 1 );
 has 'sha1'    => ( is => 'ro', isa => 'Str', required => 0, lazy_build => 1 );
 
+__PACKAGE__->meta->make_immutable;
+
 sub _build_sha1 {
     my $self = shift;
     my $sha1 = Digest::SHA1->new;
@@ -29,5 +30,4 @@ sub raw {
     return $self->kind . ' ' . $self->size . "\0" . $self->content;
 }
 
-__PACKAGE__->meta->make_immutable;
-
+1;
